package com.bwie.common.handle;

import com.bwie.common.exception.ArgumentIllegalExceptio;
import com.bwie.common.exception.auth.AuthException;
import com.bwie.common.result.Result;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.messaging.handler.annotation.support.MethodArgumentTypeMismatchException;
import org.springframework.validation.FieldError;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.RestControllerAdvice;

import javax.servlet.http.HttpServletRequest;
import javax.validation.ConstraintViolationException;
import java.util.stream.Collectors;

/**
 * @author gxb
 * @description TODO
 * @date 2023-12-26 19:14
 */
@RestControllerAdvice
public class GlodExceptionHandle {
    private static final Logger logger= LoggerFactory.getLogger(GlodExceptionHandle.class);

    /**
     * 处理Auth
     * @param request
     * @param authException
     * @return
     */
    @ExceptionHandler(AuthException.class)
    public Result authException(HttpServletRequest request,AuthException authException){
        StringBuffer requestURL = request.getRequestURL();
        logger.error("请求URL：{}，出现了异常，异常信息：{}"
        ,requestURL,authException.getMessage());
        return Result.error(authException.getMessage());
    }


    @ExceptionHandler(ArgumentIllegalExceptio.class)
    public Result argumentIllegalExceptionHandler(HttpServletRequest request, ArgumentIllegalExceptio ex) {
        String requestURI = request.getRequestURI();
        logger.error("请求URI：{}，出现了异常，异常信息：{}", requestURI, ex.getMessage());
        return Result.error(ex.getMessage());
    }
    /**
     * 自定义验证异常
     */
    @ExceptionHandler(MethodArgumentNotValidException.class)
    public Object handleMethodArgumentTypeMismatchException(MethodArgumentNotValidException e) {
        logger.error(e.getMessage(),e);
        return Result.error(
            e.getBindingResult().getFieldErrors()
                    .stream()
                    .map(FieldError::getDefaultMessage)
                    .collect(Collectors.joining(","))
        );
    }
    /**
     * 请求参数类型不匹配
     */
    @ExceptionHandler(MethodArgumentTypeMismatchException.class)
    public Result handleMethodArgumentTypeMismatchException(MethodArgumentTypeMismatchException e, HttpServletRequest request) {
        String requestURI = request.getRequestURI();
        logger.error("请求参数类型不匹配'{}',发生系统异常.", requestURI, e);
        return Result.error(
                String.format("请求参数类型不匹配，参数[%s]要求类型为：'%s'，但输入值为：'%s'"
                        , e.getCause()
                        , e.getRootCause().getCause()
                        , e.getCause()));
    }
    /**
     * 自定义验证异常
     */
    @ExceptionHandler(ConstraintViolationException.class)
    public Object handleConstraintViolationException(ConstraintViolationException e) {
        logger.error(e.getMessage(), e);
        return Result.error(e.getMessage());
    }

}
