package com.bwie.gateway.filters;

import com.bwie.common.constants.TokenConstants;
import com.bwie.common.utils.JwtUtils;
import com.bwie.common.utils.StringUtils;
import com.bwie.gateway.config.IgnoreWhiteConfig;
import com.bwie.gateway.utils.GatewayUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cloud.gateway.filter.GatewayFilterChain;
import org.springframework.cloud.gateway.filter.GlobalFilter;
import org.springframework.core.Ordered;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.http.server.reactive.ServerHttpRequest;
import org.springframework.stereotype.Component;
import org.springframework.web.server.ServerWebExchange;
import reactor.core.publisher.Mono;

import java.util.List;
import java.util.concurrent.TimeUnit;

/**
 * @author gxb
 * @description TODO
 * @date 2023-12-14 19:37
 */
@Component
public class AuthFilters implements GlobalFilter, Ordered {
    @Autowired
    private StringRedisTemplate redisTemplate;
    @Autowired
    private IgnoreWhiteConfig ignoreWhiteConfig;
    @Override
    public Mono<Void> filter(ServerWebExchange exchange, GatewayFilterChain chain) {
        /**
         * 获取白名单
         */
        List<String> whites = ignoreWhiteConfig.getWhites();
        /**
         * 获取请求路径
         */
        ServerHttpRequest request = exchange.getRequest();
        String path = request.getURI().getPath();
        /**
         * 打印
         */
        System.out.println("路径>>>>>>>>>>>>>"+path);
        /**
         * 判断是否存在白名单
         */
        boolean matches = StringUtils.matches(path, whites);
        /**
         * 放行
         */
        if (matches){
            return chain.filter(exchange);
        }
        /**
         * 判断token是否为空
         */
        String token = request.getHeaders().getFirst(TokenConstants.TOKEN);
        /**
         * 非空判断
         */
        if (null==token){
            return GatewayUtils.errorResponse(exchange,"未携带token");
        }
        /**
         * 解析token
         */
        try {
            JwtUtils.parseToken(token);
        } catch (Exception e) {
           return GatewayUtils.errorResponse(exchange,"token格式错误");
        }
        /**
         * 判断token时效性
         */
        String userKey = JwtUtils.getUserKey(token);
        if (!redisTemplate.hasKey(TokenConstants.LOGIN_TOKEN_KEY+userKey)){
            return GatewayUtils.errorResponse(exchange,"token过期了");
        }
        /**
         * 续期
         */
        redisTemplate.expire(TokenConstants.LOGIN_TOKEN_KEY+userKey,15, TimeUnit.MINUTES);
        /**
         * 放行
         */
        return chain.filter(exchange);
    }

    @Override
    public int getOrder() {
        return 0;
    }
}