package com.bwie.user.service.impl;

import com.alibaba.fastjson.JSON;
import com.bwie.common.constants.TokenConstants;
import com.bwie.common.domain.Give;
import com.bwie.common.domain.Packet;
import com.bwie.common.domain.User;
import com.bwie.common.domain.request.PacketRequest;
import com.bwie.common.domain.request.RedPacketRequest;
import com.bwie.common.domain.request.SendPacket;
import com.bwie.common.exception.auth.AuthException;
import com.bwie.common.result.Result;
import com.bwie.common.utils.JwtUtils;
import com.bwie.user.mapper.PacketMapper;
import com.bwie.user.service.PacketService;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.text.DecimalFormat;
import java.util.Arrays;
import java.util.List;
import java.util.Random;
import java.util.concurrent.TimeUnit;

/**
 * @author gxb
 * @description TODO
 * @date 2023-12-29 9:30
 */
@Service
public class PacketServicelmpl implements PacketService {
    @Autowired
    private PacketMapper packetMapper;
    @Autowired
    private HttpServletRequest request;
    @Autowired
    private StringRedisTemplate redisTemplate;
    //RedissonClient
//    @Autowired
//    private RedissonClient redissonClient;

    /**
     * 红包列表
     *
     * @param packetRequest
     * @return
     */
    @Override
    public Result packerList(PacketRequest packetRequest) {
        PageHelper.startPage(packetRequest.getPageNum(), packetRequest.getPageSize());
        //获取当前登录人
        String token = request.getHeader(TokenConstants.TOKEN);
        String userKey = JwtUtils.getUserKey(token);
        String s = redisTemplate.opsForValue().get(TokenConstants.LOGIN_TOKEN_KEY + userKey);
        User user = JSON.parseObject(s, User.class);
        //用户只能搜索到用户可以抢到的红包（2分）
        packetRequest.setUserId(user.getUserId());
        List<Packet> packetList = packetMapper.packList(packetRequest);
        PageInfo info = new PageInfo(packetList);
        return Result.success(info.getList(), info.getTotal() + "");
    }

    /**
     * 好友复选框
     *
     * @return
     */
    @Override
    public Result findFriends() {
        //获取当前登录人
        String token = request.getHeader(TokenConstants.TOKEN);
        String userKey = JwtUtils.getUserKey(token);
        String s = redisTemplate.opsForValue().get(TokenConstants.LOGIN_TOKEN_KEY + userKey);
        User user = JSON.parseObject(s, User.class);
        //查询好友
        List<Integer> findFriends = packetMapper.findFriends(user.getUserId());
        //查询好友
        List<User> users = packetMapper.finduserName(findFriends);
        return Result.success(users);
    }

    /**
     * 发红包
     *
     * @param sendPacket
     * @return
     */
    @Override
    public Result addPacket(SendPacket sendPacket) {
        //判断余额
        //获取当前登录人
        String token = request.getHeader(TokenConstants.TOKEN);
        String userKey = JwtUtils.getUserKey(token);
        String s = redisTemplate.opsForValue().get(TokenConstants.LOGIN_TOKEN_KEY + userKey);
        User user = JSON.parseObject(s, User.class);
        //余额
        double newMoney = user.getMoney();
        double packetPrice = sendPacket.getPacketPrice();

        if (packetPrice > newMoney) {
            throw new AuthException("余额不足！！！");
        }
        sendPacket.setUserId(user.getUserId());
        //添加红包表
        Integer i = packetMapper.InterPacket(sendPacket);
        if (i > 0) {
            //添加中间表
            List<Integer> userIds = sendPacket.getUserIds();
            Integer packetId = sendPacket.getPacketId();
            packetMapper.InterGive(packetId, userIds);
            //修改monry余额
            packetMapper.updateUserMoney(user.getUserId(), packetPrice);
            //查询当前值
            User endMoney = packetMapper.selectEndMoney(user.getUserId());
            user.setMoney(endMoney.getMoney());
            redisTemplate.opsForValue().set(TokenConstants.LOGIN_TOKEN_KEY + userKey, JSON.toJSONString(user));
            //存缓存   key:红包ID 值：红包金额  过期时间：红包持续时间
            redisTemplate.opsForValue().set(packetId + "", sendPacket.getPacketPrice() + "", sendPacket.getPacketDuration(), TimeUnit.MINUTES);
            return Result.success("发送成功");
        }
        return Result.error("添加失败");
    }

    /**
     * 抢红包（阿里路亚）（牛逼了哦）（哦呦）（傻波一）（哈皮波斯带）（什么玩意）（牛逼轰轰）（张耀东）
     *
     * @param redPacketRequest
     * @return
     */
    @Override
    public Result redPacket(RedPacketRequest redPacketRequest) {
        //判断红包时间是否过期
        if (!redisTemplate.hasKey(redPacketRequest.getPacketId() + "")) {
            throw new AuthException("红包已过期!!!!!!!");
        }
        //获取当前登录人
        String token = request.getHeader(TokenConstants.TOKEN);
        String userKey = JwtUtils.getUserKey(token);
        String s = redisTemplate.opsForValue().get(TokenConstants.LOGIN_TOKEN_KEY + userKey);
        User user = JSON.parseObject(s, User.class);
        //检查是否抢到
        Give give = packetMapper.findTrue(user.getUserId(), redPacketRequest.getPacketId());
        if (give.getGiveState() != 1) {
            throw new AuthException("已抢过~~~~~");
        }
        //查询红包个数
        Packet packet = packetMapper.findPackCount(redPacketRequest.getPacketId());
        if (packet.getPacketCount() == 0) {
            throw new AuthException("红包已抢完~~~~~");
        }
        //红包已领取个数
        Integer packetClai = packet.getPacketClai();
        //红包总个数
        Integer packetCount = packet.getPacketCount();
        //查询剩余个数
        Integer newCount =  packetCount - packetClai;

        //红包金额
        Double packetPrice = packet.getPacketPrice();
        //剩余金额
        Double packetRema = packet.getPacketRema();
        //发红包的金额
        Double newPrice = packetPrice-packetRema;
        //创建锁对象
//        RLock lock = redissonClient.getLock("one");

        //获取锁
//        try {
//            lock.tryLock(5,TimeUnit.MINUTES);


        //保留后两位
        DecimalFormat decimalFormat = new DecimalFormat("#.##");
        String format = decimalFormat.format(newPrice);
        double endPrice = Double.parseDouble(format);
        //储存每个红包的金额
        double[] redPacket = new double[newCount];
        //创建随机数
        Random random = new Random();
        //循环
        for (int i = 0; i < newCount - 1; i++) {
            double amout = random.nextDouble() * (endPrice / newCount) * 2;
            redPacket[i] = amout;
        }
        redPacket[newCount - 1] = endPrice;
        Arrays.sort(redPacket);

        //释放锁
//                if (lock.isHeldByCurrentThread()){
//                    if (lock.isLocked()){
//                        lock.unlock();
//                    }
//                }
        //保留后两位
        DecimalFormat decimalFormats = new DecimalFormat("#.##");
        String formats = decimalFormats.format(redPacket[0]);
        double v1 = Double.parseDouble(formats);
        //修改红包余额
        //余额等于   红包金额 - 抢到的红包金额   num = 5

        double newVv = endPrice - redPacket[0];

        //最后一个包
        if (1==newCount){
            Packet pa = packetMapper.findPacketCounts(redPacketRequest.getPacketId());
            //剩余金额
            Double packetRema1 = pa.getPacketRema();
            redPacketRequest.setPacketRema(0.00);
            redPacketRequest.setPacketPrice(packetRema1);
            packetMapper.UpdateRem(redPacketRequest);
            //修改红包抢到状态   修改中间表
            packetMapper.UpdateGiveState(redPacketRequest.getPacketId(), user.getUserId());
            //修改用户余额
            packetMapper.UpdateYue(redPacket[0],user.getUserId());
            //修改红包状态
            packetMapper.UpdatePacketSang(pa.getPacketId());
            return Result.success("抢红包成功，恭喜您喜得" + packetRema1 + "，元");
        }



        redPacketRequest.setPacketRema(newVv);
        redPacketRequest.setPacketPrice(redPacket[0]);

        packetMapper.UpdateRem(redPacketRequest);
        //修改红包抢到状态   修改中间表
        packetMapper.UpdateGiveState(redPacketRequest.getPacketId(), user.getUserId());
        //修改用户余额
        packetMapper.UpdateYue(redPacket[0],user.getUserId());

        return Result.success("抢红包成功，恭喜您喜得" + v1 + "，元");


//
//        } catch (InterruptedException e) {
//            throw new RuntimeException(e);
//        }


        //释放锁
//        if (lock.isHeldByCurrentThread()){
//            if (lock.isLocked()){
//                lock.unlock();
//            }
//        }
    }

    /**
     * 定时器扫描  返回余额
     */
    @Scheduled(cron = "0/10 * * * * ?")
    public void take() {
        System.out.println("当当当当>>>>>>定时器启动......轰隆隆！轰隆隆！轰隆隆！轰隆隆！");
        //查询红包表
        List<Packet> packets = packetMapper.selectPacket();

        if (!packets.isEmpty()) {
            for (Packet packet : packets) {

                //问题：怎么解决累计加
                if (!redisTemplate.hasKey(packet.getPacketId() + "")) {
                    //修改红包状态
                    packetMapper.UpdatePacketState(packet.getPacketId());
                    //查询剩余金额
                    Packet packet1 = packetMapper.selectPacketById(packet.getPacketId());
                    double packetRema = packet1.getPacketRema();
                    if (packetRema > 0.01) {
                        //退还用户余额
                        packetMapper.updateaddUserMoney(packet.getUserId(), packetRema);
                        //修改红包退回状态
                        packetMapper.UpdateRPacketRequest(packet.getPacketId());
                    }
                }
            }
        }
    }

}
