package com.bwie.user.service.impl;

import com.alibaba.fastjson.JSON;
import com.bwie.common.constants.TokenConstants;
import com.bwie.common.domain.Fie;
import com.bwie.common.domain.Friend;
import com.bwie.common.domain.Procut;
import com.bwie.common.domain.User;
import com.bwie.common.domain.request.AddUser;
import com.bwie.common.domain.request.Conditit;
import com.bwie.common.domain.request.Inqri;
import com.bwie.common.exception.auth.AuthException;
import com.bwie.common.result.Result;
import com.bwie.common.utils.JwtUtils;
import com.bwie.user.mapper.UserMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import javax.swing.*;
import java.util.*;
import java.util.stream.Collectors;

import static java.lang.reflect.Array.set;

/**
 * @author gxb
 * @description TODO
 * @date 2023-12-19 15:32
 */
@Service
public class UserServicelmpl implements UserService{
    @Autowired
    private UserMapper userMapper;
    @Autowired
    private StringRedisTemplate redisTemplate;
    @Autowired
    private HttpServletRequest request;
    @Override
    public Result<User> findName(String username) {
        User user = userMapper.findName(username);
        return Result.success(user);
    }

    /**
     * 发送验证码
     * @param tel
     * @return
     */
    @Override
    public Result sendCode(String tel) {
        if (!tel.matches("^1[3-9]\\d{9}$")){
           throw new AuthException("手机号格式错误");
        }
        String code = "";
        for (int i = 0; i < 6; i++) {
            code+=new Random().nextInt(10);
        }
        redisTemplate.opsForValue().set(tel,code);
        return Result.success("验证码为："+code+"，请十分钟内使用~","发送成功");
    }

    /**
     * 注册
     * @param addUser
     * @return
     */
    @Override
    public Result Sign(AddUser addUser) {
        if (!redisTemplate.hasKey(addUser.getTel())){
            throw new AuthException("验证码已过期，请重新获取");
        }
        String getCode = redisTemplate.opsForValue().get(addUser.getTel());
        if (!getCode.equals(addUser.getCode())){
            throw new AuthException("验证码错误");
        }
        //添加
        Integer i = userMapper.Sign(addUser);
        if (i<0){
            throw new AuthException("注册失败");
        }
        return Result.success("注册成功");
    }

    /**
     * 好友页面
     * @param conditit
     * @return
     */
    @Override
    public Result FriedList(Conditit conditit) {
        //获取当前登录人
        String token = request.getHeader(TokenConstants.TOKEN);
        String userKey = JwtUtils.getUserKey(token);
        String s = redisTemplate.opsForValue().get(TokenConstants.LOGIN_TOKEN_KEY + userKey);
        User user = JSON.parseObject(s, User.class);
        //查询好友Id
        List<Integer> friendId = userMapper.FriendId(user.getUserId());
        conditit.setUserIds(friendId);
        //查询好友列表
        List<User> userList = userMapper.FriedList(conditit);
        //用户登录需要，把我和我的好友存放到redis当中，使用set数据结构
        for (User user1 : userList) {
            redisTemplate.opsForValue().set("haoyou",JSON.toJSONString(user1));
        }
        return Result.success(userList,"好友列表");
    }

    /**
     * 删除好友
     * @param userId
     * @return
     */
    @Override
    public Result deleUser(Integer userId) {
        //获取当前登录人
        String token = request.getHeader(TokenConstants.TOKEN);
        String userKey = JwtUtils.getUserKey(token);
        String s = redisTemplate.opsForValue().get(TokenConstants.LOGIN_TOKEN_KEY + userKey);
        User user = JSON.parseObject(s, User.class);
        Integer i =  userMapper.deleUser(userId,user.getUserId());
        Integer o =  userMapper.deleUsers(userId,user.getUserId());
        if (i<0){
            throw new AuthException("删除失败");
        }
        return Result.success("删除成功");
    }

    /**
     * 添加好友申请表
     * @param inqri
     * @return
     */
    @Override
    public Result Addfriends(Inqri inqri) {
        //获取当前登录人
        String token = request.getHeader(TokenConstants.TOKEN);
        String userKey = JwtUtils.getUserKey(token);
        String s = redisTemplate.opsForValue().get(TokenConstants.LOGIN_TOKEN_KEY + userKey);
        User user = JSON.parseObject(s, User.class);
        //查询好友Id
        List<Integer> friendId = userMapper.FriendId(user.getUserId());
        //查询所有用户
        List<Integer> userList = userMapper.findAllUser();

        //将两个列表转换为集合
        Set<Integer> set1 = new HashSet<>(friendId);
        Set<Integer> set2 = new HashSet<>(userList);

        //获取两个集合的不重复值
        set2.removeAll(set1);
        List<Integer> newUserIds = new ArrayList<Integer>(set2);

        System.out.println("去重后的Ids"+newUserIds);

        //塞值
        inqri.setUserIds(newUserIds);
        List<User> users = userMapper.findAddAllUser(inqri);
        return Result.success(users,"添加好友列表");
    }

    /**
     * 申请添加好友
     * @param userId
     * @return
     */
    @Override
    public Result FriedRequest(Integer userId) {
        //获取当前登录人
        String token = request.getHeader(TokenConstants.TOKEN);
        String userKey = JwtUtils.getUserKey(token);
        String s = redisTemplate.opsForValue().get(TokenConstants.LOGIN_TOKEN_KEY + userKey);
        User user = JSON.parseObject(s, User.class);

        //查询是否存在
        Friend friend = userMapper.findCf(user.getUserId(),userId);
        if (friend!=null){
            throw new AuthException("该用户已经是您的好友");
        }
        //申请
        Integer i = userMapper.InsertFei(user.getUserId(),userId);
        if (i<0){
            throw new AuthException("申请失败");
        }
        //修改状态
        Fie fie = userMapper.findFieState(user.getUserId(),userId);
        //修改
        userMapper.updFriedState(fie.getFeiState(),userId);
        return Result.success("已申请");
    }

    /**
     * 查询好友申请表
     * @return
     */
    @Override
    public Result FieList() {
        //获取当前登录人
        String token = request.getHeader(TokenConstants.TOKEN);
        String userKey = JwtUtils.getUserKey(token);
        String s = redisTemplate.opsForValue().get(TokenConstants.LOGIN_TOKEN_KEY + userKey);
        User user = JSON.parseObject(s, User.class);
        //查询
        List<Integer> Id = userMapper.FieList(user.getUserId());
        //查询列表
        List<User> userList = userMapper.findAddAll(Id);
        int size = userList.size();
        return Result.success(userList,size+"");
    }

    /**
     * 查询所有哦
     * @return
     */
    @Override
    public Result<List<Procut>> AllProcut() {
        List<Procut> procutList = userMapper.AllProcut();
        return Result.success(procutList);
    }

    /**
     * 过滤父类
     * @return
     */
    @Override
    public Result subclass() {
        Result<List<Procut>> listResult = this.AllProcut();
        List<Procut> procutList = listResult.getData();
         List<Procut> proll = procutList.stream()
                 .filter(procut -> procut.getTreeId()==0)
                 .peek(procut -> procut.setChilren(getChilren(procut,procutList).isEmpty()?null:getChilren(procut,procutList)))
                 .collect(Collectors.toList());
         return Result.success(proll);


    }

    /**
     * 修改申请好友状态
     * @param feiId
     * @param state
     * @return
     */
    @Override
    public Result updateFriedRequest(Integer feiId, Integer state) {
            userMapper.updateFriedRequest(feiId,state);
            //查询
        Fie fie = userMapper.findUserId(feiId);
        Integer userId = fie.getUserId();
        Integer id = fie.getId();
        //添加好友
        userMapper.addFriend(userId,id);
        userMapper.addFriends(id,userId);

        return Result.success("成功！！！！！！");
    }

    /**
     * 批量删除
     * @param ids
     * @return
     */
    @Override
    public Result deleteFriedRequest(Integer[] ids) {
        userMapper.deleteFriedRequest(ids);
        return Result.success("删除成功");
    }



    private List<Procut> getChilren(Procut p,List<Procut> procutList){
            List<Procut> proAll = procutList.stream()
                    .filter(procut -> procut.getTreeId().intValue()==p.getProcutId())
                    .peek(procut -> procut.setChilren(getChilren(procut,procutList).isEmpty()?null:getChilren(procut,procutList)))
                    .collect(Collectors.toList());
            return proAll;
    }
}
